"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImageControls = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const slate_react_1 = require("slate-react");
const status_bar_1 = require("./status-bar");
const remove_button_1 = require("./remove-button");
const resize_controls_1 = require("./resize-controls");
const hosted_image_context_1 = require("./hosted-image-context");
/**
 * Display image controls that appear over the image.
 *
 * It works by displaying the actual image inside the image controls which is
 * rendered in its children.
 *
 * A `<span>` is placed surrounding the image such that it is flushed tight to
 * the image and has `position: relative;`.
 *
 * Now that we have a tight container, we can relatively position things like
 * the <ProgressBar /> and the <ErrorMessage /> as well as the <RemoveIcon />
 * and the <ResizeControls />.
 */
function ImageControls({ element, children: image, }) {
    const { origin, size } = (0, hosted_image_context_1.useHostedImageContext)();
    const focused = (0, slate_react_1.useFocused)();
    const selected = (0, slate_react_1.useSelected)();
    const showResizeControls = focused && selected;
    const STATUS_BAR_MARGIN = 16;
    const statusBarWidth = size.width - STATUS_BAR_MARGIN * 2;
    return ((0, jsx_runtime_1.jsxs)("span", { draggable: true, contentEditable: false, style: {
            position: "relative",
            display: "inline-block",
            /**
             * This is required so that we don't get an extra gap at the bottom.
             * When display is 'inline-block' we get some extra space at the bottom
             * for the descenders because the content is expected to co-exist with text.
             *
             * Setting vertical-align to top, bottom or middle fixes this because it is
             * no longer baseline which causes the issue.
             *
             * This is usually an issue with 'img' but also affects this scenario.
             *
             * https://stackoverflow.com/questions/5804256/image-inside-div-has-extra-space-below-the-image
             *
             * Also, make sure that <img> on the inside is display: 'block'.
             */
            verticalAlign: "top",
            /**
             * Disable user select. We use our own selection display.
             */
            userSelect: "none",
        }, children: [image, (0, jsx_runtime_1.jsx)(status_bar_1.StatusBar, { upload: origin, width: statusBarWidth, height: 16, style: {
                    position: "absolute",
                    top: "50%",
                    marginTop: -6,
                    left: 16,
                    right: 16,
                    boxShadow: "0 0 3px 0px rgba(0,0,0,1)",
                } }), origin.status === "error" ? (0, jsx_runtime_1.jsx)(remove_button_1.RemoveButton, { element: element }) : null, showResizeControls ? (0, jsx_runtime_1.jsx)(resize_controls_1.ResizeControls, { element: element }) : null] }));
}
exports.ImageControls = ImageControls;
